--- radeontool.c.orig	Wed Feb 11 04:50:27 2004
+++ radeontool.c	Mon Aug 22 13:09:41 2005
@@ -21,7 +21,12 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <sys/mman.h>
+#ifdef __LINUX__
 #include <asm/page.h>
+#elif __FreeBSD__
+#include <machine/param.h>
+#include <sys/pciio.h>
+#endif
 
 #include "radeon_reg.h"
 
@@ -233,6 +238,7 @@
 }
 
 
+#ifdef __LINUX__
 /* Here we fork() and exec() the lspci command to look for the Radeon hardware address. */
 static void map_radeon_cntl_mem(void)
 {
@@ -318,6 +324,72 @@
         printf("Radeon found. Base control address is %x.\n",base);
     radeon_cntl_mem = map_devince_memory(base,0x2000);
 }
+#endif
+
+#ifdef __FreeBSD__
+#define MAX_RADEON_CARDS 10
+static void map_radeon_cntl_mem(void)
+{
+    int fd ;
+    struct pci_conf_io pci;
+    struct pci_match_conf pmc;
+    struct pci_conf pc[MAX_RADEON_CARDS];
+    struct pci_io pi;
+    
+    pmc.pc_vendor = 0x1002;/*ATI*/
+    pmc.pc_class = 0x3; /*Display*/
+    pci.offset =0;
+    pmc.flags = PCI_GETCONF_MATCH_VENDOR|PCI_GETCONF_MATCH_CLASS;
+    pci.pat_buf_len = sizeof(pmc);
+    pci.num_patterns = 1;
+    pci.patterns =&pmc;
+    pci.match_buf_len = sizeof(pc);
+    pci.num_matches=0;
+    pci.matches = pc;
+
+    if(skip >= MAX_RADEON_CARDS ){
+        fprintf(stderr, "Cannot handle more than %d devices\n", MAX_RADEON_CARDS);
+	exit(-1);
+    }
+
+    fd = open("/dev/pci", O_RDWR);
+    if(fd == -1){
+        perror("open");
+	exit(-1);
+    }
+
+    if(ioctl(fd, PCIOCGETCONF, &pci) == -1){
+        perror("PCIOCGETCONF");
+	exit(-1);
+    }
+
+    if(pci.num_matches == 0){
+        fprintf(stderr, "No ATI Radeon devices found\n");
+	exit(-1);
+    }
+
+    if(pci.num_matches < skip ){
+        fprintf(stderr, "Found only %d ATI Radeon device(s)\n", pci.num_matches);
+	exit(-1);
+    }
+
+    pi.pi_sel = pc[skip].pc_sel;
+    pi.pi_reg = 0x18;
+    pi.pi_width = 4;
+    if(ioctl(fd, PCIOCREAD, &pi) == -1){
+        perror("PCIOCREAD");
+	exit(-1);
+    }
+    
+    if(pi.pi_data &1){
+        fprintf(stderr, "Not memory mapped\n");
+	exit(-1);
+    }
+
+    radeon_cntl_mem = map_devince_memory(pi.pi_data&0xfffffff0,0x2000);
+}
+
+#endif
 
 int main(int argc,char *argv[]) 
 {
@@ -329,7 +401,7 @@
         debug=1;
         argv++; argc--;
     };
-    if(strcmp(argv[1],"--skip=") == 0) {
+    if(argc > 1 && strncmp(argv[1],"--skip=", 7) == 0) {
         skip=atoi(argv[1]+7);
         argv++; argc--;
     };
